SystemOrganization addCategory: #'Spec-Debugger'!

!PreDebugWindow methodsFor: '*spec-debugger' stamp: 'AndreiChis 12/13/2012 12:25'!
spawnSpecDebugger
	model spawnSpecDebugger! !

Debugger subclass: #SpecProxyDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Debugger'!

!SpecProxyDebugger class methodsFor: 'class initialization' stamp: 'AndreiChis 12/13/2012 11:46'!
initialize
	super initialize.

	"Add ourselves to registry. See [Smalltalk tools]" 
	Smalltalk tools register: self as: #debugger! !

!SpecProxyDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2012 11:46'!
debuggerClass
	^ SpecDebugger ! !

!SpecProxyDebugger methodsFor: 'initialize' stamp: 'AndreiChis 12/13/2012 12:26'!
preDebugButtonQuads
	| buttons |
	self isPostMortem 
		ifTrue: [ ^ self postMortemPreDebugButtonQuads ].
	^ {
		{'Proceed' translated.	#proceed. 	#blue. 	'Continue execution' translated}.
		{'Abandon' translated.	#abandon. 	#black.	'Abandon this execution by closing this window' translated}.
		{'Debug'    		translated.		#debug.		#red. 	'Bring up a debugger' translated}.
		{'Spec Debug' translated. #spawnSpecDebugger. #red. 'Bring up the Spec Debugger' translated}
	}! !

!SpecProxyDebugger methodsFor: 'notifier menu' stamp: 'AndreiChis 1/28/2013 11:36'!
spawnSpecDebugger
	| topView |
	topView := self topView.
	topView model: nil.
	"so close won't release me."
	self breakDependents.
	topView delete.
	
	self debuggerClass debugSession: ((self debuggerClass 
		spanNewSessionForContext: self interruptedContext fromProcess: self interruptedProcess)
			errorWasInUIProcess: errorWasInUIProcess) ! !

ComposableModel subclass: #SpecDebugger
	instanceVariableNames: 'session stack code toolbar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Debugger'!

!SpecDebugger class methodsFor: 'instance creation' stamp: 'AndreiChis 1/28/2013 11:33'!
debugContext: aContext fromProcess: aProcess

	self debugSession: (self spanNewSessionForContext: aContext fromProcess: aProcess)
! !

!SpecDebugger class methodsFor: 'instance creation' stamp: 'AndreiChis 1/28/2013 11:32'!
debugSession: aSession

	<primitive: 19> "simulation guard"

	WorldState addDeferredUIMessage: [

		self new 
			session: aSession; 
			openWithSpec;
			ensureBehavior;
			yourself
		]
! !

!SpecDebugger class methodsFor: 'private' stamp: 'AndreiChis 1/7/2013 10:38'!
sessionClass

	^ DebugSession ! !

!SpecDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 1/28/2013 11:29'!
spanNewSessionForContext: aContext fromProcess: aProcess

	^ self sessionClass process: aProcess context: aContext ! !

!SpecDebugger class methodsFor: 'specs' stamp: 'AndreiChis 12/6/2012 16:34'!
spec
	<spec: #default>
	
	^ SpecLayout composed
		newColumn: [ :column | 
			column
				add: #stack;
				add: #toolbar height: 25;
				addSplitter;
				add: #code ]! !

!SpecDebugger methodsFor: 'private' stamp: 'AndreiChis 1/28/2013 10:58'!
close

	self removeActionsForSession: self session.
	self session terminate.
	 ! !

!SpecDebugger methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/7/2012 17:31'!
code

	^ code! !

!SpecDebugger methodsFor: 'private' stamp: 'AndreiChis 12/6/2012 16:21'!
columnsFor: anEntry
	| stream | 
	
	stream := ReadWriteStream on: ''.
	stream 
		nextPutAll: anEntry receiver class name asString.
	
	anEntry receiver class == anEntry methodClass
		ifFalse: [ 
			stream
				nextPutAll: '(';
				nextPutAll: anEntry methodClass name asString ;
				nextPutAll: ')'].

	"^ {stream contents . anEntry selector}"
	^ String streamContents: [:s | anEntry printDebugOn: s ]! !

!SpecDebugger methodsFor: 'accessing' stamp: 'AndreiChis 1/28/2013 11:00'!
context

	^ self session context! !

!SpecDebugger methodsFor: 'private' stamp: 'AndreiChis 1/28/2013 10:58'!
ensureBehavior

	self code behavior: self code behavior.
	self code setSelection: (self session selectedCodeRange: self stack selectedItem) ! !

!SpecDebugger methodsFor: 'protocol' stamp: 'AndreiChis 1/28/2013 12:01'!
initialExtent 

	^  610@510! !

!SpecDebugger methodsFor: 'initialization' stamp: 'AndreiChis 1/28/2013 11:02'!
initialize

	session := nil asValueHolder.
	super initialize.
	
	session whenChangedDo: [:newSession :oldSession |
		self removeActionsForSession: oldSession.
		self registerActionsForSession: newSession.
		self updateToolbarFromSession: newSession.
		self updateStackFromSession: newSession ].! !

!SpecDebugger methodsFor: 'initialization' stamp: 'AndreiChis 12/9/2012 21:01'!
initializePresenter
	
	self stack whenListChanged: [
		self stack setSelectedItem: self context ].

	self stack whenSelectedItemChanged: [:aContext |
		self toolbar selectedContext: aContext.
		self updateCodeFromContext: aContext ]! !

!SpecDebugger methodsFor: 'initialization' stamp: 'AndreiChis 12/7/2012 18:02'!
initializeWidgets 

	self instantiateModels: {
		#stack -> #ListComposableModel. 
		#code -> #TextModel.
		#toolbar -> #SpecDebuggerToolbar }.

	self stack displayBlock: [:e | self columnsFor: e ].
	self code aboutToStyle: true.! !

!SpecDebugger methodsFor: 'protocol' stamp: 'AndreiChis 12/6/2012 17:56'!
openWithSpec 

	| w |
	w := super openWithSpec.
	w whenClosedDo: [ self close ].
	^ w! !

!SpecDebugger methodsFor: 'private' stamp: 'AndreiChis 1/28/2013 10:59'!
registerActionsForSession: aSession

	aSession ifNotNil: [
		aSession
			when: #restart send: #updateRestart to: self;
			when: #resume send: #updateResume to: self;
			when: #stepInto send: #updateStepInto to: self;
			when: #stepOver send: #updateStepOver to: self;
			when: #stepThrough send: #updateStepThrough to: self ]! !

!SpecDebugger methodsFor: 'private' stamp: 'AndreiChis 1/28/2013 10:59'!
removeActionsForSession: aSession

	aSession ifNotNil: [ 
		aSession  
			removeActionsForEvent: #restart;
			removeActionsForEvent: #resume;
			removeActionsForEvent: #stepInto;
			removeActionsForEvent: #stepOver;
			removeActionsForEvent: #stepThrough ]. ! !

!SpecDebugger methodsFor: 'accessing' stamp: 'AndreiChis 1/28/2013 11:00'!
session

	^ session contents! !

!SpecDebugger methodsFor: 'accessing' stamp: 'AndreiChis 1/28/2013 11:00'!
session: aSession

	session contents: aSession ! !

!SpecDebugger methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 21:05'!
stack

	^ stack! !

!SpecDebugger methodsFor: 'protocol' stamp: 'AndreiChis 12/6/2012 18:07'!
title

	^ 'Spec Debugger'! !

!SpecDebugger methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 21:05'!
toolbar

	^ toolbar! !

!SpecDebugger methodsFor: 'updating' stamp: 'AndreiChis 1/28/2013 11:03'!
updateCodeFromContext: aContext 

	aContext 
		ifNil: [
			self code 
				text: '';
				behavior: nil ]
		ifNotNil: [
			self code
				text: aContext sourceCode;
				behavior: aContext receiver class;
				setSelection: (self session selectedCodeRange: aContext) ]! !

!SpecDebugger methodsFor: 'updating actions' stamp: 'AndreiChis 12/7/2012 17:27'!
updateRestart

	self updateStep 
	! !

!SpecDebugger methodsFor: 'updating actions' stamp: 'AndreiChis 12/7/2012 17:26'!
updateResume

	! !

!SpecDebugger methodsFor: 'updating' stamp: 'AndreiChis 1/28/2013 11:03'!
updateStackFromSession: aSession

	self stack items: aSession stack! !

!SpecDebugger methodsFor: 'updating' stamp: 'AndreiChis 1/28/2013 11:03'!
updateStep

	self updateStackFromSession: self session.
	
! !

!SpecDebugger methodsFor: 'updating actions' stamp: 'AndreiChis 12/6/2012 17:05'!
updateStepInto

	self updateStep ! !

!SpecDebugger methodsFor: 'updating actions' stamp: 'AndreiChis 12/6/2012 17:04'!
updateStepOver

	self updateStep 
	! !

!SpecDebugger methodsFor: 'updating actions' stamp: 'AndreiChis 12/7/2012 17:26'!
updateStepThrough

	self updateStep ! !

!SpecDebugger methodsFor: 'updating' stamp: 'AndreiChis 1/28/2013 11:04'!
updateToolbarFromSession: aSession

	self toolbar session: aSession.! !

ComposableModel subclass: #SpecDebuggerToolbar
	instanceVariableNames: 'session selectedContext resumeButton restartButton stepIntoButton stepOverButton stepThroughButton runToSelectionButton whereIsButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Debugger'!

!SpecDebuggerToolbar class methodsFor: 'specs' stamp: 'AndreiChis 1/28/2013 11:51'!
spec
	<spec>
	
	^ SpecLayout composed
		newRow: [ :r | 
			r
				add: #resumeButton;
				add: #restartButton;
				add: #stepIntoButton;
				add: #stepOverButton;
				add: #stepThroughButton;
				add: #runToSelectionButton;
				add: #whereIsButton ]! !

!SpecDebuggerToolbar methodsFor: 'accessing' stamp: 'AndreiChis 1/28/2013 11:06'!
currentContext

	^ self selectedContext isNil 
		ifTrue: [ self session context ]
		ifFalse: [ self selectedContext ]! !

!SpecDebuggerToolbar methodsFor: 'private' stamp: 'AndreiChis 1/28/2013 12:00'!
debugger

	^  owner! !

!SpecDebuggerToolbar methodsFor: 'initialization' stamp: 'AndreiChis 1/28/2013 11:07'!
initialize

	selectedContext := nil asValueHolder.
	session := nil asValueHolder.
	super initialize! !

!SpecDebuggerToolbar methodsFor: 'initialization' stamp: 'AndreiChis 1/28/2013 11:50'!
initializeWidgets 

	self instantiateModels: {
		#resumeButton -> #ButtonModel.
		#restartButton -> #ButtonModel.
		#stepIntoButton -> #ButtonModel.
		#stepOverButton -> #ButtonModel.
		#stepThroughButton -> #ButtonModel.
		#runToSelectionButton -> #ButtonModel.
		#whereIsButton -> #ButtonModel }.
		
	self resumeButton
		label: 'Proceed';
		state: false;
		action: [ self resumeAction ].
	self restartButton
		label: 'Restart';
		state: false;
		action: [ self restartAction ].
	self stepIntoButton
		label: 'Into';
		state: false;
		action: [ self stepIntoAction ].
	self stepOverButton
		label: 'Over';
		state: false;
		action: [ self stepOverAction ].
	self stepThroughButton  
		label: 'Through';
		state: false;
		action: [ self stepThroughAction ].
	self runToSelectionButton  
		label: 'Run to Here';
		state: false;
		action: [ self runToSelectionAction ].
	self whereIsButton  
		label: 'Where';
		state: false;
		action: [ self whereIsAction ].! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 1/28/2013 11:07'!
restartAction

	self session restart: self currentContext ! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 20:55'!
restartButton

	^ restartButton! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 1/28/2013 11:07'!
resumeAction

	self session 
		resume;
		clear.
	self window delete.! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 20:55'!
resumeButton

	^ resumeButton! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 1/28/2013 12:09'!
runToSelectionAction
	|selectionInterval|	

	selectionInterval := self debugger code getSelection.	
	selectionInterval first > selectionInterval last ifTrue: [ ^ self ].
	self session 
		runToSelection: selectionInterval 
		inContext: self currentContext
! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 1/28/2013 11:57'!
runToSelectionButton

	^ runToSelectionButton! !

!SpecDebuggerToolbar methodsFor: 'accessing' stamp: 'AndreiChis 12/6/2012 16:39'!
selectedContext
	
	^ selectedContext contents ! !

!SpecDebuggerToolbar methodsFor: 'accessing' stamp: 'AndreiChis 12/6/2012 16:40'!
selectedContext: aContext
	
	selectedContext contents: aContext! !

!SpecDebuggerToolbar methodsFor: 'accessing' stamp: 'AndreiChis 1/28/2013 11:06'!
session
	
	^ session contents ! !

!SpecDebuggerToolbar methodsFor: 'accessing' stamp: 'AndreiChis 1/28/2013 11:07'!
session: aSession
	
	session contents: aSession! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 1/28/2013 11:07'!
stepIntoAction

	self session stepInto: self currentContext ! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 20:55'!
stepIntoButton

	^ stepIntoButton! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 1/28/2013 11:07'!
stepOverAction

	self session stepOver: self currentContext ! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 20:56'!
stepOverButton

	^ stepOverButton ! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 1/28/2013 11:07'!
stepThroughAction

	self session stepThrough: self currentContext ! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 12/9/2012 20:56'!
stepThroughButton

	^ stepThroughButton ! !

!SpecDebuggerToolbar methodsFor: 'actions' stamp: 'AndreiChis 1/28/2013 11:58'!
whereIsAction

	self debugger code setSelection: (self session pcRangeForContext:  self currentContext)	
	! !

!SpecDebuggerToolbar methodsFor: 'accessing widgets' stamp: 'AndreiChis 1/28/2013 11:57'!
whereIsButton

	^ whereIsButton! !
SpecProxyDebugger initialize!
